package	jp.co.sds_corp.hibiya.connector;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import jp.co.sds_corp.hibiya.adapter.RMIAdapter;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* RMIを使用して{@link IChecker}の情報を取得します。
*/
public class RMIConnector extends AbstractConnector
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String	host;	// ホスト名
	int	port;	// ポート番号
	String	regName;	// Registry名
	String	user;	// ユーザ名
	String	password;	// パスワード
	Receiver	rcv;
	RMIAdapter.IReceiver remote;

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		host = config.get( "Host" );
		port = Integer.valueOf( config.get( "Port" )).intValue();
		regName = config.get( "Name" );
		user = config.get( "user", null );
		password = config.get( "password", null );
	}

	protected void connect() throws Exception {
		String url = null;
		try {
			url = "rmi://" + host + ":" + port + "/" + regName;
			logger.debug( "connect[" + getName() + "] url[" + url + "]" );
			remote = (RMIAdapter.IReceiver )Naming.lookup( url );

			rcv = new Receiver();
			remote.getCheckerMap();
			remote.addCheckerListener( rcv );
			remote.addConnectionListener( rcv );
			fireConnected( new ConnectionEvent( this ));
			setConnected( true );
		} catch ( Exception ex ) {
			logger.error( "connect[" + getName() + "] error " + ex );
			setConnected( false );
			throw( ex );
		}
	}

	public void start() {
		try {
			connect();
		} catch ( Exception ex ) {
		}
		super.start();
	}

	public void stop() {
		super.stop();
		if ( remote == null || !isConnected()) {
			logger.debug( " stopped.[" + getName() + "]" );
			return;
		}
		try {
			remote.removeCheckerListener( rcv );
		} catch ( Exception ex ) {
			logger.error( "removeCheckerListener() error ", ex );
		}
		try {
			remote.removeConnectionListener( rcv );
		} catch ( Exception ex ) {
			logger.error( "removeConnectionListener() error ", ex );
		}
		logger.debug( " stopped.[" + getName() + "]" );
	}

	/**
	* 接続状態を検査します。
	*/
	public boolean checkConnection() throws Exception {
		if ( !isConnected()) {
			connect();
		}
		remote.healthcheck();
		return true;
	}

	/**
	* {@link IChecker}を取得します。
	* @param	name	ICheckerの登録名
	*/
	public IChecker getChecker( String name ) throws Exception {
		IChecker ret = null;
		ret = remote.getChecker( name );
		return ret;
	}

	/**
	* {@link IChecker}の一覧を返します。
	* @return	ICheckerの一覧が格納されたMap
	*/
	public Map<String,IChecker> getCheckerMap() throws Exception {
		Map<String,IChecker> ret = null;
		ret = remote.getCheckerMap();
		return ret;
	}

	/**
	* {@link IChecker}を追加します
	* @param	chk		追加するIChecker
	*/
	public void addChecker( IChecker chk ) throws Exception {
		remote.addChecker( chk );
	}

	/**
	* {@link IChecker}を削除します。
	* @param	chk		削除するIChecker
	*/
	public void removeChecker( IChecker chk ) throws Exception {
		remote.removeChecker( chk );
	}

	/**
	* ホスト名を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var;
	}
	/**
	* ホスト名を取得します。
	* @return	ホスト名
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var;
	}
	/**
	* ポート番号を取得します。
	* @return	ポート番号
	*/
	public int getPort() {
		return port;
	}

	/**
	* Registry名を設定します。
	* @param	var		設定するデータ
	*/
	public void setRegistryName( String var ) {
		this.regName = var;
	}
	/**
	* Registry名を取得します。
	* @return	Registry名
	*/
	public String getRegistryName() {
		return regName;
	}

	/**
	* ユーザ名を設定します。
	* @param	var		設定するデータ
	*/
	public void setUser( String var ) {
		this.user = var;
	}
	/**
	* ユーザ名を取得します。
	* @return	ユーザ名
	*/
	public String getUser() {
		return user;
	}

	/**
	* パスワードを設定します。
	* @param	var		設定するデータ
	*/
	public void setPassword( String var ) {
		this.password = var;
	}
	/**
	* パスワードを取得します。
	* @return	パスワード
	*/
	public String getPassword() {
		return password;
	}


	/**
	* 設定内容を保存します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Host", host );
		config.addChild( "Port", String.valueOf( port ) );
		config.addChild( "Name", regName );
	}

	public interface IReceiver extends Remote,
						ICheckerListener, IConnectionListener
	{
	}

	public class Receiver extends UnicastRemoteObject implements IReceiver
	{
		private static final long serialVersionUID = 1L;

		public Receiver() throws RemoteException, MalformedURLException {
		}

		public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
			fireCheckerChange( ev );
		}
		public void connected( ConnectionEvent ev ) {
			fireConnected( ev );
		}
		public void disconnected( ConnectionEvent ev ) {
			fireDisconnected( ev );
		}
	}

}
