package	jp.co.sds_corp.hibiya.server;

import java.text.DecimalFormat;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.device.CPUStatus;
import jp.co.sds_corp.util.device.DeviceManager;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
CPUの状況を監視します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="CPU1" class="jp.co.sds_corp.hibiya.server.CPUChecker"&gt;
	&lt;MaxLoad&gt;20&lt;/MaxLoad&gt;
</PRE>
MaxLoad要素で、CPU使用率の最大値を指定してください。load averageがこの値を超えるとエラーです。
<BR>
Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class CPUChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	DecimalFormat   df = new DecimalFormat( "#,###" );
	DecimalFormat   df2 = new DecimalFormat( "0.0#" );
	int	maxLoad;	// load averageの最大値

	public CPUChecker() {
		setType( "CPU" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		maxLoad = config.getInteger( "MaxLoad" );
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			CmnDate startTime = new CmnDate();
			CPUStatus info = DeviceManager.getCPUStatus( 1 );
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			putResult( "UserTime", df.format( info.getUserTime() ));
			putResult( "SystemTime", df.format( info.getSystemTime() ));
			putResult( "IdleTime", df.format( info.getIdleTime() ));
			putResult( "CPUName", info.getName() );
			putResult( "Vendor", info.getVendor() );
			putResult( "Speed", info.getSpeed() );
			putResult( "Kernel", info.getKernel() );
			putResult( "Load Average 1min", df2.format( info.getAvg1min()));
			putResult( "Load Average 5min", df2.format( info.getAvg5min()));
			putResult( "Load Average 15min", df2.format( info.getAvg15min()));

			if ( isDebug()) {
				logger.debug( "loadavg "
					+ " 1min=" + info.getAvg1min()
					+ " 5min=" + info.getAvg5min()
					+ " 15min=" + info.getAvg15min() );
			}
			if ( info.getAvg1min() > (double )maxLoad ||
				info.getAvg5min() > (double )maxLoad ||
				info.getAvg15min() > (double )maxLoad )
			{
				setException( new Exception( "overload"
								+ " load=" + df2.format( info.getAvg1min() )
								+ "," + df2.format( info.getAvg5min() )
								+ "," + df2.format( info.getAvg15min() )
								));
				setCheckerStatus( IChecker.Status.NG );
			} else {
				setCheckerStatus( IChecker.Status.OK );
			}
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	/**
	* load averageの最大値(maxLoad)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMaxLoad( int var ) {
		this.maxLoad = var;
	}
	/**
	* load averageの最大値(maxLoad)を取得します。
	* @return	maxLoadの内容
	*/
	public int getMaxLoad() {
		return maxLoad;
	}


	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "MaxLoad", String.valueOf( maxLoad ));
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "max=" + maxLoad + ";"
			+ "]";
		return ret;
	}

}
