package	jp.co.sds_corp.hibiya.server;

import java.net.InetAddress;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.net.Ping;

/**
Networkの状況をICMPを使用して監視します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="Ping1" class="jp.co.sds_corp.hibiya.server.PingChecker"&gt;
	&lt;Name&gt;192.168.0.1&lt;/Name&gt;
	&lt;Address&gt;192.168.0.1&lt;/Address&gt;
	&lt;Retry&gt;3&lt;/Retry&gt;
</PRE>

Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class PingChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	transient Ping	ping;
	String	address;	// IPアドレス
	int	retry;	// リトライ回数

	public PingChecker() {
		setType( "Ping" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		setRetry( config.getInteger( "Retry", 1 ));
		setAddress( config.get( "Address", null ));
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		Throwable exception = null;
		CmnDate startTime = new CmnDate();
		boolean bRet = false;
		int end = ( retry > 0 ) ? retry : 1;
		for ( int i = 0; i < end; i++ ) {
			try {
				ping = new Ping( InetAddress.getByName( address ));
				bRet = ping.check();
				if ( bRet ) {
					break;
				}
			} catch ( Throwable ex ) {
				logger.debug( "retry [" + address + "]" );
				exception = ex;
			} finally {
				if ( ping != null ) {
					ping.close();
				}
			}
		}
		CmnDate endTime = new CmnDate();
		clearResult();
		setResponseTime( startTime, endTime );

		if ( bRet ) {
			setCheckerStatus( IChecker.Status.OK );
		} else {
			setCheckerStatus( IChecker.Status.NG );
			synchronized( this ) {
				setException( exception );
			}
		}
	}

	/**
	* IPアドレス(address)を設定します。
	* @param	var		設定するデータ
	*/
	public void setAddress( String var ) {
		this.address = var ;
	}
	/**
	* IPアドレス(address)を取得します。
	* @return	addressの内容
	*/
	public String getAddress() {
		return address;
	}

	/**
	* リトライ回数を設定します。
	* @param	var		設定するデータ
	*/
	public void setRetry( int var ) {
		this.retry = var;
	}
	/**
	* リトライ回数を取得します。
	* @return	リトライ回数
	*/
	public int getRetry() {
		return retry;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Address", address );
		if ( retry > 1 ) {
			config.addChild( "Retry", String.valueOf( retry ) );
		}
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "[";
		ret += "Address=" + address + ";";
		if ( retry > 1 ) {
			ret += "Retry=" + retry;
		}
		ret += "]";
		return ret;
	}

}
