package	jp.co.sds_corp.hibiya.server;

import java.io.InputStream;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
Webサーバの監視を行います。
<BR>
XMLは以下の形式です。<BR>
Port,Method,HTTPVersionは、オプションです。
<PRE>
&lt;Service name="Web1" class="jp.co.sds_corp.hibiya.server.WebChecker"&gt;
	&lt;Host&gt;www.sds-corp.co.jp&lt;/Host&gt;
	&lt;Port&gt;80&lt;/Port&gt;
	&lt;Method&gt;GET&lt;/Method&gt;
	&lt;URL&gt;/&lt;/URL&gt;
	&lt;HTTPVersion&gt;1.0&lt;/HTTPVersion&gt;
	&lt;username&gt;user&lt;/username&gt;
	&lt;password&gt;pwd&lt;/password&gt;
	&lt;Timeout&gt;5000&lt;/Timeout&gt;
&lt;/Service&gt;
&lt;Service name="Web1" class="jp.co.sds_corp.hibiya.server.WebChecker"&gt;
	&lt;URL&gt;http://www.sds-corp.co.jp/&lt;/URL&gt;
	&lt;username&gt;user&lt;/username&gt;
	&lt;password&gt;pwd&lt;/password&gt;
	&lt;Timeout&gt;5000&lt;/Timeout&gt;
&lt;/Service&gt;
</PRE>

Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class WebChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	private	int		nTimeout = 10;
	String	host;
	String	port;
	String	url;
	String	method;
	String	HTTPVersion;
	String	username;	// 認証に使用するユーザ名
	String	password;	// 認証に使用するパスワード
	String	cookie;

	int		timeout;

	public WebChecker() {
		setType( "Web" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		setHost( config.get( "Host", null ));
		setPort( config.get( "Port", null ));
		setURL( config.get( "URL", null ));
		setMethod( config.get( "Method", null ));
		setHTTPVersion( config.get( "HTTPVersion", null ));
		setUsername( config.get( "Username", null ));
		setPassword( config.get( "Password", null ));
		setTimeout( config.getInteger( "Timeout", 5000 ));
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			String	host = getHost();
			URL url;

			if ( getHost() == null || getHost().length() == 0 ) {
				url = new URL( getURL() );
			} else {
				String surl = "http://" + host;
				if ( port != null ) {
					int		nPort = Integer.valueOf( getPort() ).intValue();
					surl += ":" + nPort;
				}
				if ( getURL() != null ) {
					surl += getURL();
				}
				url = new URL( surl );
			}
			if ( username != null && username.length() > 0 ) {
				HttpAuthenticator http_auth =
					new HttpAuthenticator( username, password );
				Authenticator.setDefault( http_auth );
			}

			CheckThread thr = new CheckThread( url, cookie );
			CmnDate startTime = new CmnDate();
			thr.start();
			try {
				thr.join( getTimeout() );
			} catch ( Exception ex ) {
			}
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			if ( thr.isAlive()) {
				thr.interrupt();
				Thread.yield();
			}
			if ( !thr.isOk()) {
				if ( thr.getException() == null ) {
					throw( new InterruptedException( "interrupted" ));
				} else {
					setException( thr.getException() );
				}
			}

			HttpURLConnection con = thr.getConnection();
			if ( con != null ) {
				int nRes = con.getResponseCode();
				String sc = con.getHeaderField( "Set-Cookie" );
				if ( sc != null ) {
					this.cookie = sc;
				}

				putResult( "ResponseCode", String.valueOf( nRes ));
				putResult( "ResponseMessage", con.getResponseMessage() );
				Map<String,List<String>> map = con.getHeaderFields();
				Iterator<String> itr = map.keySet().iterator();
				while ( itr.hasNext()) {
					String key = (String )itr.next();
					if ( key != null ) {
						putResult( key, con.getHeaderField( key ));
					}
				}
			}

			setCheckerStatus( IChecker.Status.OK );
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	private class CheckThread extends Thread
	{
		URL	url;
		Exception ex;
		HttpURLConnection con;
		boolean bOk = false;
		String	cookie;

		protected CheckThread( URL url, String cookie ) {
			this.url = url;
			this.cookie = cookie;
		}

		public void run() {
			try {
				if ( isDebug()) {
					logger.debug( "check=" + url.toString() );
				}
				con = (HttpURLConnection)url.openConnection();
				setConnection( con );
				con.setAllowUserInteraction( true );
				if ( getMethod() != null && getMethod().length() > 0 ) {
					con.setRequestMethod( getMethod() );
				}
				if ( cookie != null ) {
					con.setRequestProperty( "Cookie", cookie );
				}
			} catch ( Exception ex ) {
				setException( ex );
				con = null;
				return;
			}
			try {
				con.connect();
				InputStream is = con.getInputStream();
				is.close();

				setOk( true );
			} catch ( Exception ex ) {
				setException( ex );
			} finally {
				try {
					con.disconnect();
				} catch ( Exception ex ) {
				}
			}
		}

		protected synchronized boolean isOk() {
			return bOk;
		}
		protected synchronized void setOk( boolean var ) {
			bOk = var;
		}
		protected synchronized void setException( Exception var ) {
			this.ex = var;
		}
		protected synchronized HttpURLConnection getConnection() {
			return this.con;
		}
		protected synchronized void setConnection( HttpURLConnection var ) {
			this.con = var;
		}
		protected synchronized Exception getException() {
			return this.ex;
		}
	}

	class HttpAuthenticator extends Authenticator {
		private String username;
		private String password;
		public HttpAuthenticator( String username, String password ) {
			this.username = username;
			this.password = password;
		}
		protected PasswordAuthentication getPasswordAuthentication() {
			return new
				PasswordAuthentication( username, password.toCharArray());
		}
	}

	/**
	* Webサーバのホスト名(host)を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var ;
	}
	/**
	* Webサーバのホスト名(host)を取得します。
	* @return	hostの内容
	*/
	public String getHost() {
		return host;
	}

	/**
	* Webサーバのポート番号(port)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( String var ) {
		this.port = var ;
	}
	/**
	* Webサーバのポート番号(port)を取得します。
	* @return	portの内容
	*/
	public String getPort() {
		return port;
	}

	/**
	* URL(url)を設定します。
	* @param	var		設定するデータ
	*/
	public void setURL( String var ) {
		this.url = var ;
	}
	/**
	* URL(url)を取得します。
	* @return	urlの内容
	*/
	public String getURL() {
		return url;
	}

	/**
	* Method(method)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMethod( String var ) {
		this.method = var ;
	}
	/**
	* Method(method)を取得します。
	* @return	methodの内容
	*/
	public String getMethod() {
		return method;
	}

	/**
	* HTTP version(HTTPVersion)を設定します。
	* @param	var		設定するデータ
	*/
	public void setHTTPVersion( String var ) {
		this.HTTPVersion = var ;
	}
	/**
	* HTTP version(HTTPVersion)を取得します。
	* @return	HTTPVersionの内容
	*/
	public String getHTTPVersion() {
		return HTTPVersion;
	}

	/**
	* 認証に使用するユーザ名を設定します。
	* @param	var		設定するデータ
	*/
	public void setUsername( String var ) {
		this.username = var;
	}
	/**
	* 認証に使用するユーザ名を取得します。
	* @return	認証に使用するユーザ名
	*/
	public String getUsername() {
		return username;
	}

	/**
	* 認証に使用するパスワードを設定します。
	* @param	var		設定するデータ
	*/
	public void setPassword( String var ) {
		this.password = var;
	}
	/**
	* 認証に使用するパスワードを取得します。
	* @return	認証に使用するパスワード
	*/
	public String getPassword() {
		return password;
	}

	/**
	* タイムアウト(timeout)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTimeout( int var ) {
		this.timeout = var ;
	}
	/**
	* タイムアウト(timeout)を取得します。
	* @return	timeoutの内容
	*/
	public int getTimeout() {
		return timeout;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( host == null || host.length() == 0 ) {
			config.addChild( "URL", url );
		} else {
			config.addChild( "Host", host );
			if ( port != null && port.length() > 0 ) {
				config.addChild( "Port", port );
			}
			if ( url != null && url.length() > 0 ) {
				config.addChild( "URL", url );
			}
			if ( HTTPVersion != null && HTTPVersion.length() > 0 ) {
				config.addChild( "HTTPVersion", HTTPVersion );
			}
		}
		if ( method != null && method.length() > 0 ) {
			config.addChild( "Method", method );
		}
		if ( username != null && username.length() > 0 ) {
			config.addChild( "Username", username );
			config.addChild( "Password", password );
		}
		config.addChild( "Timeout", String.valueOf( timeout ) );
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		if ( host == null || host.length() == 0 ) {
			ret += "[URL=" + url + "]";
		} else {
			ret += "[" + "Host=" + host + ";";
			if ( port != null && port.length() > 0 ) {
				ret += "Port=" + port + ";";
			}
			if ( url != null && url.length() > 0 ) {
				ret += "URL=" + url + ";";
			}
			if ( HTTPVersion != null && HTTPVersion.length() > 0 ) {
				ret += "HTTPVersion=" + HTTPVersion + ";";
			}
		}
		if ( username != null && username.length() > 0 ) {
			ret += "Username=" + username + ";"
//				+ "Password=" + password + ";";
				+ "Password=*;";
		}
		if ( method != null ) {
			ret += "Method=" + method + ";";
		}
		ret += "Timeout=" + timeout + ";"
			+ "]";
		return ret;
	}

}
